<?php
/* Copyright (C) 2007-2015 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2017      Open-DSI             <support@open-dsi.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *	    \file       htdocs/quicklist/admin/about.php
 *		\ingroup    quicklist
 *		\brief      Page about of quicklist module
 */

// Change this following line to use the correct relative path (../, ../../, etc)
$res=0;
if (! $res && file_exists("../../main.inc.php")) $res=@include '../../main.inc.php';			// to work if your module directory is into a subdir of root htdocs directory
if (! $res && file_exists("../../../main.inc.php")) $res=@include '../../../main.inc.php';		// to work if your module directory is into a subdir of root htdocs directory
if (! $res) die("Include of main fails");
require_once DOL_DOCUMENT_ROOT.'/core/lib/admin.lib.php';
dol_include_once('/quicklist/lib/quicklist.lib.php');
dol_include_once('/quicklist/core/modules/modQuickList.class.php');

$langs->load("admin");
$langs->load("quicklist@quicklist");
$langs->load("opendsi@quicklist");

if (!$user->admin) accessforbidden();


/**
 * View
 */

$wikihelp='EN:QuickList_En|FR:QuickList_Fr|ES:QuickList_Es';
llxHeader('', $langs->trans("QuickListSetup"), $wikihelp);

$linkback='<a href="'.DOL_URL_ROOT.'/admin/modules.php">'.$langs->trans("BackToModuleList").'</a>';
print load_fiche_titre($langs->trans("QuickListSetup"),$linkback,'title_setup');
print "<br>\n";


$head=quicklist_admin_prepare_head();
$isV14p = version_compare(DOL_VERSION, "14.0.0") >= 0;

if ($isV14p) {
    print dol_get_fiche_head($head, 'about', $langs->trans("Module163011Name"), 0, 'opendsi@quicklist');
} else {
    dol_fiche_head($head, 'about', $langs->trans("Module163011Name"), 0, 'opendsi@quicklist');
}

$modClass = new modQuickList($db);
$constantLastVersion = !empty($modClass->getVersion()) ? $modClass->getVersion() : 'NC';

$supportvalue = "/*****"."<br>";
$supportvalue.= " * Module : ".$langs->trans("Module163011Name")."<br>";
$supportvalue.= " * Module version : ".$constantLastVersion."<br>";
$supportvalue.= " * Dolibarr version : ".DOL_VERSION."<br>";
$supportvalue.= " * Dolibarr version installation initiale : ".$conf->global->MAIN_VERSION_LAST_INSTALL."<br>";
$supportvalue.= " * Option colonne sélection à gauche : ".(isset($conf->global->MAIN_CHECKBOX_LEFT_COLUMN) && (!empty($conf->global->MAIN_CHECKBOX_LEFT_COLUMN)) ? $conf->global->MAIN_CHECKBOX_LEFT_COLUMN : 0)."<br>";
$supportvalue.= " * Version PHP : ".PHP_VERSION."<br>";
$supportvalue.= " *****/"."<br>";
$supportvalue.= "Description de votre problème :"."<br>";

// print '<div class="div-table-responsive-no-min">';
print '<table class="centpercent">';

//print '<tr class="liste_titre"><td colspan="2">' . $langs->trans("Authors") . '</td>';
//print '</tr>'."\n";

// Easya Solutions
print '<tr>';
print '<form id="ticket" method="POST" target="_blank" action="https://support.easya.solutions/create_ticket.php">';
print '<input name=message type="hidden" value="'.$supportvalue.'" />';
print '<input name=email type="hidden" value="'.$user->email.'" />';
print '<td class="titlefield center"><img alt="Easya Solutions" src="../img/opendsi_dolibarr_preferred_partner.png" /></td>'."\n";
print '<td class="left"><p>'.$langs->trans("OpenDsiAboutDesc1").' <button type="submit" >'.$langs->trans("OpenDsiAboutDesc2").'</button> '.$langs->trans("OpenDsiAboutDesc3").'</p></td>'."\n";
print '</tr>'."\n";

print '</table>'."\n";

print dol_get_fiche_end();


llxFooter();

$db->close();
