<?php
/* Copyright (C) 2017      Open-DSI             <support@open-dsi.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

//require_once DOL_DOCUMENT_ROOT.'/core/lib/functions.lib.php';
require_once DOL_DOCUMENT_ROOT.'/core/lib/date.lib.php';
require_once DOL_DOCUMENT_ROOT.'/core/class/html.form.class.php';
dol_include_once('/quicklist/class/quicklist.class.php');
dol_include_once('/quicklist/lib/quicklist.lib.php');

/**
 *  \file       htdocs/quicklist/class/actions_quicklist.class.php
 *  \ingroup    quicklist
 *  \brief      File for hooks
 */

class ActionsQuickList
{
	/**
	 * @var DoliDB Database handler.
	 */
	public $db;
	/**
	 * @var string Error
	 */
	public $error = '';
	/**
	 * @var array Errors
	 */
	public $errors = array();

	/**
	 * @var array Hook results. Propagated to $hookmanager->resArray for later reuse
	 */
	public $results = array();

	/**
	 * @var string String displayed by executeHook() immediately after return
	 */
	public $resprints;

	/**
	 * @var array
	 */
	private $invalid_params = ['token', 'confirm', 'formfilteraction',
		'button_search_x', 'button_search.x', 'button_search',
		'button_removefilte_x', 'button_removefilter.x', 'button_removefilter',
		'action', 'massaction', 'confirmmassaction', 'checkallactions', 'toselect[]',
		'button_quicklist_addfilter_x', 'button_quicklist_addfilter.x', 'button_quicklist_addfilter',
		'filter_name', 'filter_scope', 'filter_scope_usergroup', 'filter_update_url', 'filter_menu', 'filter_id',
		'filter_hashtag', 'filter_default', 'mainmenu', 'leftmenu', 'page', 'confirmmassactioninvisible',
	];

	/**
	 * @var array
	 */
	private $authorized_invalid_params = [ 'token', 'confirm', 'formfilteraction',
		'action', 'massaction', 'confirmmassaction', 'checkallactions', 'toselect[]',
		'button_quicklist_addfilter_x', 'button_quicklist_addfilter.x', 'button_quicklist_addfilter',
		'filter_name', 'filter_scope', 'filter_scope_usergroup', 'filter_update_url', 'filter_menu',
		'filter_hashtag', 'filter_default', 'page', 'sortfield', 'sortorder',
	];

	/**
	 * @var array
	 */
	private $ids_params = ['id', 'socid', 'contact_id', 'ref', 'lineid', 'idmenu'];

	/**
	 * @var User[user_id]						List of users cached
	 */
	public static $users_cached;


	/**
	 * Constructor
	 *
	 * @param DoliDB $db Database handler
	 */
	public function __construct($db)
	{
		$this->db = $db;
	}

	/**
	 * Overloading the doActions function : replacing the parent's function with the one below
	 *
	 * @param	array 			$parameters		Hook metadatas (context, etc...)
	 * @param	CommonObject	$object			The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param	string			$action			Current action (if set). Generally create or edit or null
	 * @param	HookManager		$hookmanager	Hook manager propagated to allow calling another hook
	 * @return	int                             Return integer < 0 on error, 0 on success, 1 to replace standard code
	 */
	public function doActions($parameters, &$object, &$action, $hookmanager)
	{
		global $langs, $user;

		$langs->load('quicklist@quicklist');

		$quicklist = new QuickList($this->db);
		$act = GETPOST('action', 'alphanohtml');

		//--------------------------------------------------------------------
		// Get all infos
		//--------------------------------------------------------------------
		$result = $this->getInfos($parameters, $quicklist, $context, $act, $params_url, $ids_params_url, $invalid_params_url);
		if (!$result) {
			return 0;
		}

		//--------------------------------------------------------------------
		// Add filter
		//--------------------------------------------------------------------
		if ($act == 'quicklist_addfilter_confirm' && !($quicklist->id > 0)) {
			$fk_menu = null;
			if (!empty(GETPOST("filter_menu", 'alpha'))) {
				// TODO create menu
				$fk_menu = null;
			}

			$quicklist->name = GETPOST("filter_name", 'alpha');
			$quicklist->page_context = $context;
			$quicklist->params = $params_url;
			$quicklist->scope = GETPOST("filter_scope", 'alpha');
			$quicklist->fk_menu = $fk_menu;
			$quicklist->default = GETPOST("filter_default", 'alpha') ? 1 : 0;
			$quicklist->hash_tag = urldecode(GETPOST("filter_hashtag", 'alpha'));
			$result = $quicklist->create($user);
			if ($result < 0) {
				setEventMessages($langs->trans('QuickListFilterSavedError', $quicklist->errorsToString()), null, 'errors');
				$_GET['button_quicklist_addfilter'] = true; // simulate click to go back to filter button
			}

			if ($result) {
				$usergroup_ids = $quicklist->scope == QuickList::QUICKLIST_SCOPE_USERGROUP ? explode(',', GETPOST("filter_scope_usergroup")) : array();
				$quicklist->set_usergroup($usergroup_ids);

				setEventMessages($langs->trans('QuickListFilterSaved'), null);
				header('Location: ' . $_SERVER["PHP_SELF"] . "?filter_id=" . $quicklist->id . (!empty($params_url) ? '&' . $params_url : '') . (!empty($ids_params_url) ? '&' . $ids_params_url : '') . $quicklist->hash_tag);
				exit;
			}
		}
		//--------------------------------------------------------------------
		// Edit filter
		//--------------------------------------------------------------------
		elseif ($act == 'quicklist_editfilter_confirm' && $quicklist->id > 0 && (!empty($user->admin) || $user->hasRight('quicklist', 'manage') || $quicklist->fk_user_author == $user->id)) {
			$fk_menu = null;
			if (!empty(GETPOST("filter_menu", 'alpha'))) {
				// TODO update url menu
				if (!empty(GETPOST("filter_update_url", 'alpha'))) {
				}
				$fk_menu = $quicklist->fk_menu;
			} else {
				// TODO delete menu
			}

			$quicklist->name = GETPOST("filter_name", 'alpha');
			if (!empty(GETPOST("filter_update_url", 'alpha'))) {
				$quicklist->params = $params_url;
				$quicklist->hash_tag = urldecode(GETPOST("filter_hashtag", 'alpha'));
			}
			$quicklist->scope = GETPOST("filter_scope", 'alpha');
			$quicklist->fk_menu = $fk_menu;
			$quicklist->default = GETPOST("filter_default", 'alpha') ? 1 : 0;
			$result = $quicklist->update($user);
			if ($result < 0) {
				setEventMessages($langs->trans('QuickListFilterSavedError', $quicklist->errorsToString()), null, 'errors');
				$_GET['button_quicklist_addfilter'] = true; // simulate click to go back to filter button
			}

			if ($result) {
				$usergroup_ids = $quicklist->scope == QuickList::QUICKLIST_SCOPE_USERGROUP ? explode(',', GETPOST("filter_scope_usergroup")) : array();
				$quicklist->set_usergroup($usergroup_ids);

				setEventMessages($langs->trans('QuickListFilterSaved'), null);
				header('Location: ' . $_SERVER["PHP_SELF"] . "?filter_id=" . $quicklist->id . (!empty($params_url) ? '&' . $params_url : '') . (!empty($ids_params_url) ? '&' . $ids_params_url : '') . $quicklist->hash_tag);
				exit;
			}
		}
		//--------------------------------------------------------------------
		// Delete filter
		//--------------------------------------------------------------------
		elseif ($act == 'quicklist_deletefilter_confirm' && $quicklist->id > 0 && (!empty($user->admin) || $user->hasRight('quicklist', 'manage') || $quicklist->fk_user_author == $user->id)) {
			// TODO delete menu
			$result = $quicklist->delete($user);
			if ($result < 0) {
				setEventMessages($langs->trans('QuickListFilterDeletedError', $quicklist->errorsToString()), null, 'errors');
			}

			if ($result) {
				setEventMessages($langs->trans('QuickListFilterDeleted'), null);
				header('Location: ' . $_SERVER["PHP_SELF"] . "?filter_id=" . $quicklist->id . (!empty($params_url) ? '&' . $params_url : '') . (!empty($ids_params_url) ? '&' . $ids_params_url : '') . $quicklist->hash_tag);
				exit;
			}
		}
		//--------------------------------------------------------------------
		// Set filter
		//--------------------------------------------------------------------
		elseif ($act == 'quicklist_select') {
			$_SESSION['quicklist_filter_applied'][$context] = true;
			$location_params = array();
			if (!empty($quicklist->params)) {
				$location_params[] = $this->substituteTag($quicklist->params);
			}
			if (!empty($ids_params_url)) {
				$location_params[] = $ids_params_url;
			}
			if (!empty($invalid_params_url)) {
				$location_params[] = $invalid_params_url;
			}
			header('Location: ' . $_SERVER["PHP_SELF"] . '?' . implode('&', $location_params));
			exit;
		}

		if (isset($_SESSION['quicklist_filter_applied'][$context])) {
			unset($_SESSION['quicklist_filter_applied'][$context]);
		}
		return 0;
	}

	/**
	 * Overloading the printFieldListOption function : replacing the parent's function with the one below
	 *
	 * @param	array			$parameters		Hook metadatas (context, etc...)
	 * @param	CommonObject	$object			The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param	string			$action			Current action (if set). Generally create or edit or null
	 * @param	HookManager		$hookmanager	Hook manager propagated to allow calling another hook
	 * @return	int                             Return integer <0 on error, 0 on success, 1 to replace standard code
	 */
	public function printFieldListOption($parameters, &$object, &$action, $hookmanager)
	{
		global $conf, $langs, $user;

		$langs->load('quicklist@quicklist');

		$quicklist = new QuickList($this->db);
		$act = GETPOST("action", 'alpha');
		$addfilter = GETPOST("button_quicklist_addfilter_x") || GETPOST("button_quicklist_addfilter.x") || GETPOST("button_quicklist_addfilter");

		//--------------------------------------------------------------------
		// Get all infos
		//--------------------------------------------------------------------
		$this->getInfos($parameters, $quicklist, $context, $act, $params_url, $ids_params_url, $invalid_params_url);

		//--------------------------------------------------------------------
		// Base url
		//--------------------------------------------------------------------
		$base_url = $_SERVER["PHP_SELF"] . (!empty($ids_params_url) ? "?" . $ids_params_url : '') . (!empty($params_url) ? (!empty($ids_params_url) ? '&' : '?') . $params_url : '');

		//--------------------------------------------------------------------
		// Formulaire de Ajout / modification / confirmation de suppression
		//--------------------------------------------------------------------
		if ($act == 'quicklist_editfilter' || $act == 'quicklist_deletefilter' || $addfilter) {
			$form = new Form($this->db);
			dol_include_once('/quicklist/class/html.formquicklist.class.php');
			$formquicklist = new FormQuickList($this->db);

			// Get groups of the user
			//require_once DOL_DOCUMENT_ROOT . '/user/class/usergroup.class.php';
			//$usergroup = new UserGroup($this->db);
			//$usergroups = $usergroup->listGroupsForUser($user->id);
			//if (!is_array($usergroups)) {
			//	$this->error = $usergroup->error;
			//	$this->errors = $usergroup->errors;
			//	return -1;
			//}

			//--------------------------------------------------------------------
			// Get filter
			//--------------------------------------------------------------------
			$quicklist->fetch_usergroup();
			// default values
			if (GETPOSTISSET('filter_name')) {
				$quicklist->name = GETPOST('filter_name', 'alpha');
			}
			if (GETPOSTISSET('filter_scope')) {
				$quicklist->scope = GETPOST('filter_scope', 'alpha');
			}
			if (GETPOSTISSET('filter_default')) {
				$quicklist->default = GETPOST('filter_default', 'alpha') ? 1 : 0;
			}
			if (GETPOSTISSET('filter_scope')) {
				$quicklist->usergroups = array();
				$usergroup_ids = $quicklist->scope == QuickList::QUICKLIST_SCOPE_USERGROUP ? explode(',', GETPOST('filter_scope_usergroup')) : array();
				foreach ($usergroup_ids as $usergroup_id) {
					$quicklist->usergroups[$usergroup_id] = $usergroup_id;
				}
			}

			//--------------------------------------------------------------------
			// Formulaire de Ajout / modification
			//--------------------------------------------------------------------
			if ($addfilter || ($act == 'quicklist_editfilter' && $quicklist->id > 0)) {
				$formConfirmHeight = 350;
				$formquestion = array(
					array(
						'name' => 'filter_id',
						'type' => 'hidden',
						'value' => $quicklist->id
					),
					array(
						'name' => 'filter_scope',
						'type' => 'hidden',
						'value' => (isset($quicklist->scope) ? $quicklist->scope : QuickList::QUICKLIST_SCOPE_PRIVATE)
					),
					array(
						'name' => 'filter_hashtag',
						'type' => 'hidden',
						'value' => ''
					),
					array(
						'tdclass' => 'fieldrequired',
						'name' => 'filter_name',
						'label' => $langs->trans('QuickListFilterName'),
						'type' => 'text',
						'value' => (isset($quicklist->name) ? $quicklist->name : '')
					),
					array(
						'name' => 'filter_scope_r',
						'label' => $langs->trans('QuickListFilterScope'),
						'type' => 'radio',
						'values' => array(
							QuickList::QUICKLIST_SCOPE_PRIVATE => '<label for="filter_scope_' . QuickList::QUICKLIST_SCOPE_PRIVATE . '">' . $langs->trans('QuickListScopePrivate') . '</label>',
							QuickList::QUICKLIST_SCOPE_USERGROUP => '<label for="filter_scope_' . QuickList::QUICKLIST_SCOPE_USERGROUP . '">' . $langs->trans('QuickListScopeUserGroup') . '</label>',
							QuickList::QUICKLIST_SCOPE_PUBLIC => '<label for="filter_scope_' . QuickList::QUICKLIST_SCOPE_PUBLIC . '">' . $langs->trans('QuickListScopePublic') . '</label>',
						)
					),
					array(
						'name' => 'filter_scope_usergroup',
						'label' => $langs->trans('QuickListFilterScopeUserGroup'),
						'type' => 'other',
						'value' => $formquicklist->multiselect_dolgroups((isset($quicklist->usergroups) ? array_keys($quicklist->usergroups) : array()), 'filter_scope_usergroup')
						//, '', 0, array_keys($usergroups))
					),
					array(
						'name' => 'filter_default',
						'label' => $langs->trans('QuickListFilterDefault'),
						'type' => 'checkbox',
						'value' => !empty($quicklist->default)
					),
				);

				$outJSLineList = array();
				if ($act == 'quicklist_editfilter') {
					$tagNum = 0;
					$paramList = array();
					parse_str($quicklist->params, $paramList);
					foreach ($paramList as $paramKey => $paramValue) {
						$tagMatches = array();
						// it's a date to search
						if (is_string($paramValue) && preg_match('/{{(CD|CW|CM|CY)(?:(\+|-)(D|W|M|Y)(\d+))?}}/', $paramValue, $tagMatches)) {
							if ($tagNum == 0) {
								$formquestion[] = array(
									'label' => '<br>'.$langs->trans('QuickListFilterTagRules'),
									'type' => 'other',
								);
							}

							$formquestion[] = array(
								'name' => $paramKey,
								'label' => '<u style="padding-left: 10px;">'.$paramKey.'</u>',
								'type' => 'text',
								'value' => $paramValue,
							);
							$outJSLine = 'jQuery("#'.$paramKey.'").val("'.$paramValue.'");';
							$outJSLine .= 'jQuery("input[name=\"'.$paramKey.'\"]").on("change", function(){;';
							$outJSLine .= '	jQuery("#'.$paramKey.'").val(this.value);';
							$outJSLine .= '});';
							$outJSLineList[] = $outJSLine;

							$tagNum++;
						}
					}
					if ($tagNum > 0) {
						$formquestion[] = array(
							'label' => '<br>',
							'type' => 'other',
						);
						$formConfirmHeight += 30;
						$formConfirmHeight += 20*$tagNum;
					}

					$formquestion[] = array(
						'name' => 'filter_update_url',
						'label' => $langs->trans('QuickListFilterUpdateUrl'),
						'type' => 'checkbox',
						'value' => true
					);
				}

				/*$formquestion[] = array(
					'name' => 'filter_menu',
					'label' => $langs->trans('QuickListFilterMenu'),
					'type' => 'checkbox',
					'value' => (empty($quicklist->fk_menu) ? false : true)
				);*/

				$form_title = $addfilter ? $langs->trans("QuickListAddFilter") : $langs->trans("QuickListEditFilter");
				$form_action = $addfilter ? "quicklist_addfilter_confirm" : "quicklist_editfilter_confirm";

				$formconfirm = $form->formconfirm($base_url, $form_title, '', $form_action, $formquestion, 'yes', 1, $formConfirmHeight, 800);
				$formconfirm .= '<script type="text/javascript" language="javascript">' . "\n";
				$formconfirm .= '$(document).ready(function () {' . "\n";
				$formconfirm .= '  $("#dialog-confirm").on("dialogopen", function(event, ui) {' . "\n";
				$formconfirm .= '    $("#dialog-confirm").val(encodeURIComponent(window.location.hash));' . "\n";
				$formconfirm .= '    $(\'input[name=filter_hashtag]\').val(encodeURIComponent(window.location.hash));' . "\n";
				$formconfirm .= '    $(\'input[name=filter_name]\').closest(\'tr\').find(\'td:first-child\').attr(\'width\', \'25%\');' . "\n";
				$formconfirm .= '    $.map($(\'input[type=radio][name=filter_scope_r]\'), function(item) { $(item).attr(\'id\', \'filter_scope_\' + $(item).val()); });' . "\n";
				$formconfirm .= '    $(\'input[type=radio][name=filter_scope_r]\').filter(\'[value=' . (isset($quicklist->scope) ? $quicklist->scope : QuickList::QUICKLIST_SCOPE_PRIVATE) . ']\').prop(\'checked\', true);' . "\n";
				if ($quicklist->scope != QuickList::QUICKLIST_SCOPE_USERGROUP) {
					$formconfirm .= '    $("#filter_scope_usergroup").closest("tr").hide();' . "\n";
				}
				$formconfirm .= '    $(\'input[type=radio][name=filter_scope_r]\').change(function() {' . "\n";
				$formconfirm .= '      $(\'input[type=hidden][name=filter_scope]\').val(this.value);' . "\n";
				$formconfirm .= '      if (this.value == ' . QuickList::QUICKLIST_SCOPE_USERGROUP . ') {' . "\n";
				$formconfirm .= '        $("#filter_scope_usergroup").closest("tr").show();' . "\n";
				$formconfirm .= '      } else {' . "\n";
				$formconfirm .= '        $("#filter_scope_usergroup").closest("tr").hide();' . "\n";
				$formconfirm .= '      }' . "\n";
				$formconfirm .= '    });' . "\n";
				$formconfirm .= '    $(\'select#filter_scope_usergroup\').closest(\'td\').find(\'li.select2-search.select2-search--inline\').addClass(\'minwidth300\');' . "\n";
				if (!empty($outJSLineList)) {
					$formconfirm .= implode('', $outJSLineList);
				}
				$formconfirm .= '  });' . "\n";
				$formconfirm .= '});' . "\n";
				$formconfirm .= '</script>' . "\n";
			}
			//--------------------------------------------------------------------
			// Formulaire de confirmation de suppression
			//--------------------------------------------------------------------
			elseif ($act == 'quicklist_deletefilter' && $quicklist->id > 0) {
				$formquestion = array(
					array(
						'name' => 'filter_id',
						'type' => 'hidden',
						'value' => $quicklist->id
					),
				);
				$formconfirm = $form->formconfirm($base_url, $langs->trans("QuickListDeleteFilter"), $langs->trans("QuickListConfirmDeleteFilter", $quicklist->name), "quicklist_deletefilter_confirm", $formquestion, 0, 1);
			}

			if (isset($formconfirm)) {
				quicklist_print_confirmform($formconfirm);
			}
		}

		//--------------------------------------------------------------------
		// Remplacement du bouton de remise à zero des filtres
		//--------------------------------------------------------------------
		// Get filter list of user
		$filters_list = $quicklist->liste_array($context);
		$filters = ['private' => [], 'usergroup' => [], 'public' => []];
		$filters_buttons = ['private' => [], 'usergroup' => [], 'public' => []];
		if (is_array($filters_list)) {
			foreach ($filters_list as $filter) {
				$author = $this->getUser($filter->fk_user_author);
				$author_name = isset($author) ? $author->login : $filter->fk_user_author;

				$has_selected_fields = strpos($filter->params, '&selectedfields=') != -1 || strpos($filter->params, '?selectedfields=') != -1;
				$tmp = array("filter_id=" . $filter->id);
				if (!empty($ids_params_url)) {
					$tmp[] = $ids_params_url;
				}
				if ($has_selected_fields) {
					$tmp[] = 'formfilteraction=listafterchangingselectedfields';
				}
				$url = $_SERVER["PHP_SELF"] . "?" . implode('&', $tmp);
				$value = [
					'id' => $filter->id,
					'name' => $filter->name . ((!empty($user->admin) || $user->hasRight('quicklist', 'manage')) && $filter->fk_user_author != $user->id ? ' ( ' . $author_name . ' )' : ''),
					'url' => $url,
					'hash_tag' => $filter->hash_tag,
					'author' => !empty($user->admin) || $user->hasRight('quicklist', 'manage') || $filter->fk_user_author == $user->id,
					'default' => !empty($filter->default),
					'selected' => !in_array($act, ['quicklist_editfilter']) && $quicklist->id == $filter->id
				];
				switch ($filter->scope) {
					case QuickList::QUICKLIST_SCOPE_PRIVATE:
						$filters['private'][] = $value;
						if (!empty($conf->global->QUICKLIST_SHOW_FILTERS_LIST_ABOVE_TABLE) && $filter->fk_user_author == $user->id) {
							$value['name'] = $filter->name;
							$value['author'] = $filter->fk_user_author == $user->id;
							$filters_buttons['private'][] = $value;
						}
						break;
					case QuickList::QUICKLIST_SCOPE_USERGROUP:
						$filters['usergroup'][] = $value;
						if (!empty($conf->global->QUICKLIST_SHOW_FILTERS_LIST_ABOVE_TABLE)) {
							$value['name'] = $filter->name;
							$value['author'] = $filter->fk_user_author == $user->id;
							$filters_buttons['usergroup'][] = $value;
						}
						break;
					case QuickList::QUICKLIST_SCOPE_PUBLIC:
						$filters['public'][] = $value;
						if (!empty($conf->global->QUICKLIST_SHOW_FILTERS_LIST_ABOVE_TABLE)) {
							$value['name'] = $filter->name;
							$value['author'] = $filter->fk_user_author == $user->id;
							$filters_buttons['public'][] = $value;
						}
						break;
				}
			}
		}

		print '<script type="text/javascript" language="javascript">' . "\n";
		print '$(document).ready(function () {' . "\n";
		print 'quicklist_replace_button_removefilter("' . str_replace('"', '\\"', $base_url) . '",' . json_encode($filters) . ');' . "\n";
		if (!empty($conf->global->QUICKLIST_SHOW_FILTERS_LIST_ABOVE_TABLE)) {
			print 'quicklist_show_filters_list_button(' . json_encode($filters_buttons) . ');' . "\n";
		}
		print '});';
		print '</script>' . "\n";

		return 0;
	}

	/**
	 * Get context of the list page
	 *
	 * @param	array		$parameters			Hook metadatas (context, etc...)
	 * @param	QuickList	$quicklist			QuickList handler
	 * @param   string		$context			Page context
	 * @param	string 		$act 				Page action
	 * @param	string 		$params_url 		Parameters url
	 * @param	string 		$ids_params_url		Parameters url ids
	 * @param	string 		$invalid_params_url	Invalid parameters in url
	 * @return	string							Context of the list page
	 */
	protected function getInfos($parameters, &$quicklist, &$context, &$act, &$params_url, &$ids_params_url, &$invalid_params_url)
	{
		global $user;

		$context = $this->getListContext($parameters['context']);
		if (empty($context))
			return 0;

		//--------------------------------------------------------------------
		// Base url
		//--------------------------------------------------------------------
		$params = array();
		if (empty($_GET['restore_lastsearch_values'])) { // Use $_GET here and not GETPOST
			foreach (array_merge($_POST, $_GET) as $key => $value) {
				if (!in_array($key, $this->invalid_params) && !in_array($key, $this->ids_params) && $value != '') {
					$params[$key] = $value;
				}
			}
		} else {
			// Code for search criteria persistence.
			// Retrieve values if restore_lastsearch_values
			$relativepathstring = $_SERVER["PHP_SELF"];
			// Clean $relativepathstring
			if (constant('DOL_URL_ROOT')) $relativepathstring = preg_replace('/^' . preg_quote(constant('DOL_URL_ROOT'), '/') . '/', '', $relativepathstring);
			$relativepathstring = preg_replace('/^\//', '', $relativepathstring);
			$relativepathstring = preg_replace('/^custom\//', '', $relativepathstring);
			if (!empty($_SESSION['lastsearch_values_' . $relativepathstring])) { // If there is saved values
				$tmp = json_decode($_SESSION['lastsearch_values_' . $relativepathstring], true);
				if (is_array($tmp)) {
					foreach ($tmp as $key => $value) {
						if (!in_array($key, $this->invalid_params) && !in_array($key, $this->ids_params) && $value != '') {
							$params[$key] = $value;
						}
					}
				}
			}
			// If there is saved contextpage, page or limit
			if (!in_array('contextpage', $this->invalid_params) && !in_array('contextpage', $this->ids_params) && !empty($_SESSION['lastsearch_contextpage_' . $relativepathstring])) {
				$params['contextpage'] = $_SESSION['lastsearch_contextpage_' . $relativepathstring];
			}
			if (!in_array('page', $this->invalid_params) && !in_array('page', $this->ids_params) && !empty($_SESSION['lastsearch_page_' . $relativepathstring])) {
				$params['page'] = $_SESSION['lastsearch_page_' . $relativepathstring];
			}
			if (!in_array('limit', $this->invalid_params) && !in_array('limit', $this->ids_params) && !empty($_SESSION['lastsearch_limit_' . $relativepathstring])) {
				$params['limit'] = $_SESSION['lastsearch_limit_' . $relativepathstring];
			} elseif (!in_array('limit', $this->invalid_params) && !in_array('limit', $this->ids_params) && !empty($_SESSION['lastsearch_limit_tmp_' . $relativepathstring])) {
				$params['limit'] = $_SESSION['lastsearch_limit_tmp_' . $relativepathstring];
			}
		}
		if (!empty($params['contextpage']) && empty($params['selectedfields'])) {
			$tmpvar = "MAIN_SELECTEDFIELDS_" . $params['contextpage'];
			if (!empty($user->conf->$tmpvar)) {
				$params['selectedfields'] = $user->conf->$tmpvar;
			}
		}
		$params_url = count($params) ? http_build_query($params, '', '&') : '';

		//--------------------------------------------------------------------
		// Get filter
		//--------------------------------------------------------------------
		$filter_id = GETPOST("filter_id", 'int');
		if ($filter_id > 0 && in_array($act, ['quicklist_select', 'quicklist_editfilter', 'quicklist_editfilter_confirm', 'quicklist_deletefilter', 'quicklist_deletefilter_confirm'])) {
			$result = $quicklist->fetch($filter_id);
			if ($result < 0) {
				setEventMessages($quicklist->error, $quicklist->errors, 'errors');
				return 0;
			}
		} elseif (!empty($params)) {
			$result = $quicklist->fetch(0, $params, $context);
			if ($result < 0) {
				setEventMessages($quicklist->error, $quicklist->errors, 'errors');
				return 0;
			}
		}
		$filter_id = $quicklist->id > 0 ? $quicklist->id : 0;
		if (empty($filter_id) && empty($params_url) && empty($act) && !isset($_SESSION['quicklist_filter_applied'][$context])) {
			$result = $quicklist->fetch_default($context);
			if ($result < 0) {
				setEventMessages($quicklist->error, $quicklist->errors, 'errors');
				return 0;
			} elseif ($result == 0) return 0;
			$act = 'quicklist_select';
		}

		//--------------------------------------------------------------------
		// params of ids for the url (cards case)
		//--------------------------------------------------------------------
		$ids_params = array();
		foreach (array_merge($_POST, $_GET) as $key => $value) {
			if (in_array($key, $this->ids_params) && $value != '') {
				$ids_params[$key] = $value;
			}
		}
		$ids_params_url = count($ids_params) ? http_build_query($ids_params, '', '&') : '';

		//--------------------------------------------------------------------
		// params of invalid for the url (cards case)
		//--------------------------------------------------------------------
		$invalid_params_url = array();
		foreach (array_merge($_POST, $_GET) as $key => $value) {
			if (in_array($key, $this->authorized_invalid_params) && $value != '' &&  $value != 'quicklist_select') {
				$invalid_params_url[$key] = $value;
			}
		}
		$invalid_params_url = count($invalid_params_url) ? http_build_query($invalid_params_url, '', '&') : '';

		return 1;
	}

	/**
	 * Get context of the list page
	 *
	 * @param   string  $context    All context
	 *
	 * @return  string				Context of the list page
	 */
	protected function getListContext($context)
	{
		$context_list = preg_grep('/(.*list$)/i', explode(':', $context));
		if (count($context_list)) {
			foreach ($context_list as $value) {
				return $value;
			}
		}

		return '';
	}

	/**
	 * Substitute tags to an url search
	 *
	 * @param	string	$params		Parameters in an url string
	 * @return 	string	Url string for searching form
	 */
	protected function substituteTag($params)
	{
		$paramList = array();

		if (!empty($params)) {
			$paramsModified = false;
			$now = dol_now();
			$currentDay = date('d', $now);
			$currentMonth = date('m', $now);
			$currentYear = date('Y', $now);

			parse_str($params, $paramList);
			foreach ($paramList as $paramKey => $paramValue) {
				$rules = array();

				// it's a date to search
				//if (preg_match_all('/\{\{(CD|CM|CY)((?:(\+|\-)(D|M|Y)(\d+))*)\}\}/', $paramValue, $rules)) {
				if (is_string($paramValue) && preg_match('/{{(CD|CW|CM|CY)(?:(\+|-)(D|W|M|Y)(\d+))?}}/', $paramValue, $rules)) {
					$rulesNb = count($rules);
					if ($rulesNb > 1) {
						$ruleStart = $rules[1]; // CD or CW or CM or CY
						$ruleOperation = '';
						$ruleUnity = '';
						$ruleUnityNb = 0;
						if (isset($rules[2])) {
							$ruleOperation = $rules[2]; // + or -
						}
						if (isset($rules[3])) {
							$ruleUnity = $rules[3]; // D or W or M or Y
						}
						if (isset($rules[4])) {
							$ruleUnityNb = $rules[4]; // Number of units
						}

						// rule to determine begin timestamp
						if ($ruleStart == 'CM') {
							$dateTimeBegin = dol_get_first_day($currentYear, $currentMonth); // 1st day of the current month
						} elseif ($ruleStart == 'CY') {
							$dateTimeBegin = dol_get_first_day($currentYear, 1); // 1st of january of the current year
						} elseif ($ruleStart == 'CW') {
							$tmp = dol_get_first_day_week($currentDay, $currentMonth, $currentYear);
							$dateTimeBegin = dol_mktime(0, 0, 0, $tmp['first_month'], $tmp['first_day'], $tmp['first_year']); // 1st day of the current week
						} else {
							$dateTimeBegin = $now; // current day by default
						}

						// operation with nb units
						$dateTimeFinal = $dateTimeBegin;
						if (in_array($ruleOperation, array('+', '-')) && in_array($ruleUnity, array('D', 'W', 'M', 'Y')) && $ruleUnityNb > 0) {
							if ($ruleOperation == '-') {
								$ruleUnityNb = -$ruleUnityNb;
							}
							$ruleUnityNb = (int) $ruleUnityNb;
							$dateTimeFinal = dol_time_plus_duree($dateTimeBegin, $ruleUnityNb, strtolower($ruleUnity));
						}

						// replace date filter
						$paramList[$paramKey] = dol_print_date($dateTimeFinal, '%d/%m/%Y');
						// add day, month and year search filters
						$paramList[$paramKey.'day'] = dol_print_date($dateTimeFinal, '%d');
						$paramList[$paramKey.'month'] = dol_print_date($dateTimeFinal, '%m');
						$paramList[$paramKey.'year'] = dol_print_date($dateTimeFinal, '%Y');
						$paramsModified = true;
					}
				}
			}

			if ($paramsModified === true) {
				$params = http_build_query($paramList, '', '&');
			}
		}

		return $params;
	}

	/**
	 *  Get user object (cached)
	 *
	 * @param	int			$user_id		User ID
	 * @return	User						null if error otherwise user handler
	 */
	public function getUser($user_id)
	{
		global $langs;

		// Clean parameters
		$user_id = $user_id > 0 ? $user_id : 0;

		// Check parameters
		if (empty($user_id)) {
			$langs->load('errors');
			$this->errors[] = $langs->trans("ErrorBadParameters");
			return null;
		}

		if (!isset(self::$users_cached[$user_id])) {
			$user = new User($this->db);
			$result = $user->fetch($user_id);
			if ($result < 0) {
				$this->errors[] = $langs->trans('User') . ' (ID : ' . $user_id . ')';
				$this->errors[] = $user->errorsToString();
				return null;
			} elseif ($result == 0) {
				$langs->load('errors');
				$this->errors[] = $langs->trans('ErrorRecordNotFound') . ' - ' . $langs->trans('User') . ' (ID : ' . $user_id . ')';
				return null;
			}

			self::$users_cached[$user_id] = $user;
		}

		return self::$users_cached[$user_id];
	}
}
